﻿using System.Collections.Generic;
using Jmelosegui.Mvc.Controls;
using Jmelosegui.Mvc.Controls.Overlays;
using Jmelosegui.Mvc.Controls.Tests;
using Moq;
using System.IO;
using System.Web.UI;
using Xunit;

namespace Mvc.Controls.Googlemap.Test
{
    public class GooglemapBuilderTests
    {
        private readonly GoogleMap map;
        private readonly GoogleMapBuilder controlBuilder;

        public GooglemapBuilderTests()
        {
            var writer = new Mock<HtmlTextWriter>(TextWriter.Null);
            map = GoogleMapTestHelper.CreateMap(writer.Object);
            controlBuilder = new GoogleMapBuilder(map);
        }

        [Fact]
        public void Disable_Zoom_On_DoubleClick()
        {
            controlBuilder.DisableDoubleClickZoom(true);
            Assert.True(map.DisableDoubleClickZoom);
        }

        [Fact]
        public void Setting_markers_sets_marker()
        {
            controlBuilder.Markers(markers => markers.Add().Longitude(42).Latitude(2));

            Assert.NotNull(map.Markers);
            Assert.True(map.Markers.Count == 1);
        }

        [Fact]
        public void Settings_if_marker_point_is_not_set_should_be_equal_to_map_center_point()
        {
            controlBuilder.Markers(markers => markers.Add());
            Assert.Equal(map.Markers[0].Latitude, map.Latitude);
            Assert.Equal(map.Markers[0].Longitude, map.Longitude);
        }

        [Fact]
        public void Settings_bind_model_to_markers()
        {
            var datasource = new List<TestObject>
                                 {
                                     new TestObject {
                                         Latitude = 20,
                                         Longitude = -80,
                                         Title = "Marker1"
                                     },
                                     new TestObject {
                                         Latitude = 21,
                                         Longitude = -80,
                                         Title = "Marker2"
                                     },
                                     new TestObject {
                                         Latitude = 22,
                                         Longitude = -80,
                                         Title = "Marker3"
                                     },
                                 };

            controlBuilder.Name("map").BindTo<TestObject, Marker>
            (
                datasource, mappings => mappings.For<TestObject>
                (
                    binding => binding.ItemDataBound
                    (
                        (marker, obj) =>
                        {
                            marker.Latitude = obj.Latitude;
                            marker.Longitude = obj.Longitude;
                            marker.Title = obj.Title;
                        }
                    )
                )
            );
            Assert.True(controlBuilder.ToComponent().Markers.Count == 3);
        }
    }

    public class TestObject
    {
        public double Latitude { get; set; }
        public double Longitude { get; set; }
        public string Title { get; set; }
    }
}
